//
//  100MilCounter.m
//  100MilCounter
//
//  Created by Brian Sheehan on Mon Jul 05 2004.
//  Derived from the NSStatusItem Tutorial by Jason Marr at:
//    http://www.cocoadevcentral.com/articles/000052.php
//  Copyright (c) 2004 eklektion.org. All rights reserved.
//

#import "100MilCounter.h"


@implementation _00MilCounter

long int myValue;								// The number!
long int myMillions, myThousands, mySingles;	// So I can parse it out with commas
long int myTimeToNextUpdate;					// Track seconds until next update
short autoFire;									// Flag set only if the main (5-min) timer fires
NSString *MyTitle;								// name & version, to be reported in app menu

// setValues: All we do here is get the singles, thousands and millions from myValue
-(void)setValues
{
	myMillions = myValue / 1000000;
    myThousands = (myValue - (myMillions * 1000000)) / 1000;
    mySingles = myValue - (myMillions * 1000000) - (myThousands * 1000);
}

// updateMyValue: downloads number from Apple's website
-(void)updateMyValue
{
	int localValue;				// Local copy of the number we get
	NSURL *url;					// The URL to the javascript file
	NSString *response;			// The contents of the javascript file
	NSScanner *scan;			// For parsing the javascript file

	url = [NSURL URLWithString: _00MILCOUNTER_URL ];
	response = [NSString stringWithContentsOfURL:url];
	scan = [NSScanner scannerWithString: response];
	
	// Try and extract the long int from the javascript file.  This comes back as
	// a single line of text looking like this:
	//		function count(){ var num = 98372325;return num; }
	while (! [scan isAtEnd] && ! [scan scanInt: &localValue]) {
		[scan setScanLocation: [scan scanLocation] + 1];
	} 
	
	if (! [scan isAtEnd] && (unsigned int)localValue >= myValue) {
		myValue = localValue;
		if (autoFire == 1) {
			myTimeToNextUpdate = _00MILCOUNTER_UPDATE_CYCLE;
		}
	
		//NSLog(@"RETVAL: good, %d (%d)", localValue);
	} else {
		if (myValue == 0 || myValue == 1) {
			myValue = -1;
		}
		//NSLog(@"RETVAL: EOS");
	}
	//NSLog(@"MYLOG: %ld", myValue);
}

// init: Set up a few values, some probably unnecessary if they're statics anyway....	
-(id)init
{
	myValue = 1;
	autoFire = 0;	
	NSDictionary *infoDict;
	NSString *MyName, *MyVersion;
	
	// Pull Name & version out of the InfoPlist.string file
	infoDict = (NSDictionary *)CFBundleGetLocalInfoDictionary(CFBundleGetMainBundle());
	MyName = [infoDict objectForKey: @"CFBundleName"];
	MyVersion = [infoDict objectForKey: @"CFBundleShortVersionString"];
	
	//... and use the info to make a name/version string for the app menu
	MyTitle = [NSString stringWithFormat: @"%@ v%@", MyName, MyVersion];
	
	[self setValues];

	// This wil be the menu item visible in the main menu bar
	statusItem = [[[NSStatusBar systemStatusBar]
				    statusItemWithLength: NSVariableStatusItemLength] retain];

	return self;
}

// awakeFromNib: Setup timers and menu-related stuff...
-(void)awakeFromNib
{
// Every 5 minutes, this will fire to get updated info from Apple.
	mainTimer = [[NSTimer scheduledTimerWithTimeInterval: (float)(_00MILCOUNTER_UPDATE_CYCLE)
	                      target: self
						  selector: @selector(timer:)
						  userInfo:nil
						  repeats:YES] retain];
// Fires every second, basically as a countdown timer
	quickTimer = [[NSTimer scheduledTimerWithTimeInterval: (1.0)
							target: self
							selector: @selector(quicktimer:)
							userInfo:nil
							repeats:YES] retain];
//////////////////////////////////////////////////////////////////////

	[statusItem setEnabled: YES];			// Are we "clickable?	
	[statusItem setHighlightMode: YES];		// Should we highlight when clicked? 
	[statusItem setMenu: theMenu];			// What menu are we using? 
	[statusItem setTitle: @"Waiting..."];   // Initially, set a string so it's not blank
	
	// Set up the "Name & Version" string in the menu
	[[theMenu itemWithTag: _00MILCOUNTER_MENU_VERSION] setTitle: MyTitle];

	// Everything ready - do an initial auto-update to get the number
	[mainTimer fire];
}

// forceUpdateMenu:  This is called if the user updates manually
-(void)forceUpdateMenu: (id)sender
{
	[self updateMenu];
}

// updateMenu: updates the mils, thous and singles, and stuffs 'em in the menu title
-(void)updateMenu
{
	// NSLog(@"MYLOG: updating menu %d", myValue);
	[statusItem setTarget: self];
	[self updateMyValue];

	// We must put a message in the menu if the number doesn't look good
	if (myValue == 0) {
		[statusItem setTitle: @"Waiting..."];
		myValue++;
	} else if (myValue < 0) {
		[statusItem setTitle: @"Offline..."];
	} else if (myValue == 1) {
		[statusItem setTitle: @"Retrying..."];
	} else {
		[self setValues];
		[statusItem setTitle: [NSString stringWithFormat: @"%03d,%03d,%03d", myMillions, myThousands, mySingles]];
	}
}

// timer:  this is the 5-minute main timer
-(void)timer:(NSTimer *)timer
{
	autoFire = 1;
	[self updateMenu];
	autoFire = 0;
	// NSLog(@"MYLOG: timer fired with value %ld", myValue);
}

// quicktimer: ... and this is out 1-second "tick" to track seconds to next update
-(void)quicktimer:(NSTimer *)quicktimer
{
	NSMutableString *s;
	myTimeToNextUpdate--;
	s = [NSMutableString stringWithFormat: @"Next update in: %02ld:%02ld", myTimeToNextUpdate / 60, myTimeToNextUpdate % 60];
	[[theMenu itemWithTag: _00MILCOUNTER_MENU_NEXT] setTitle: s];
}

// dealloc: ye olde cleanuppe
-(void)dealloc
{
	[statusItem release];
	[mainTimer invalidate];
	[mainTimer release];
	[super dealloc];
}

@end
